import { Component } from "@angular/core";
import {
  NavController,
  ToastController,
  LoadingController
} from "ionic-angular";
import { Storage } from "@ionic/storage";
import { CryptoProvider } from "../../providers/crypto/crypto";
import { SocialSharing } from "@ionic-native/social-sharing";

@Component({
  selector: "page-settings",
  templateUrl: "settings.html"
})
export class SettingsPage {
  keywords: string;
  privateKey: string;
  publicKey: string;

  constructor(
    public navCtrl: NavController,
    public toastCtrl: ToastController,
    private cryptoUtils: CryptoProvider,
    private storage: Storage,
    private loadingCtrl: LoadingController,
    private sharing: SocialSharing
  ) {
    this.loadValuesFromStorage();
  }

  async loadValuesFromStorage() {
    this.privateKey = await this.storage.get("privateKey");
    this.publicKey = await this.storage.get("publicKey");
    this.keywords = await this.storage.get("keywords");
  }

  async generateKeys() {
    const keys = await this.cryptoUtils.generateRsaKeys();

    this.publicKey = await this.cryptoUtils.extractPublicKey(keys);
    this.privateKey = await this.cryptoUtils.extractPrivateKey(keys);
  }

  save() {
    this.storage.set("publicKey", this.publicKey);
    this.storage.set("privateKey", this.privateKey);
    this.storage.set("keywords", this.keywords);

    this.showToast("Successfully saved!");
  }

  async publishPublicKey() {
    if (this.publicKey.length) {
      const loading = this.loadingCtrl.create();
      loading.present();

      await this.cryptoUtils.publishPublicKey(this.publicKey);

      loading.dismiss();
      this.showToast("Public key has been published!");
    }
  }

  exportPrivateKey() {
    if (this.privateKey.length) {
      this.sharing
        .share(this.privateKey, null, null, null)
        .then(() => console.log("Private key was exported"))
        .catch(() =>
          this.showToast(
            "Sorry! Something went wrong trying to export the private key :("
          )
        );
    } else {
      this.showToast("There is nothing to share.");
    }
  }

  private showToast(message: string) {
    const toast = this.toastCtrl.create({
      message: message,
      position: "bottom",
      duration: 3000
    });
    toast.present();
  }
}
